"use server";

import Docker from 'dockerode';

const docker = new Docker();
const CONTAINER_TIMEOUT = 60; // 容器超时时间（秒）

export default async function serverAction(
    sql: string,
    databaseType: string
): Promise<string> {
  let dockerImage: string;
  let command: string;
  let env: string[] = [];
  let container: Docker.Container;

  // 配置数据库参数
  switch (databaseType) {
    case 'sqlserver':
      process.env.SQL_SERVER_SA_PASSWORD='YourStrong!Passw0rd';
      dockerImage = 'mcr.microsoft.com/mssql/server:2019-latest';
      command = `/opt/mssql-tools18/bin/sqlcmd -S localhost -U SA -P ${process.env.SQL_SERVER_SA_PASSWORD} -C -Q "${sql}"`; // 添加 -C 参数
      env = ['ACCEPT_EULA=Y', `SA_PASSWORD=${process.env.SQL_SERVER_SA_PASSWORD}`];
      break;
    case 'mysql':
      process.env.MYSQL_ROOT_PASSWORD='YourStrong!Passw0rd';
      dockerImage = 'mysql:latest';
      command = `mysql -u root -p${process.env.MYSQL_ROOT_PASSWORD} -e "${sql}"`;
      env = [
        `MYSQL_ROOT_PASSWORD=${process.env.MYSQL_ROOT_PASSWORD}`,
        'MYSQL_TCP_PORT=3306',
        'MYSQL_ROOT_HOST=%',
      ];
      break;
    default:
      throw new Error('Unsupported database type');
  }

  try {
    // 创建并启动容器（SQL Server专用配置）
    container = await docker.createContainer({
      Image: dockerImage,
      Env: databaseType === 'sqlserver'
        ? [...env, 'MSSQL_AGENT_ENABLED=True', 'MSSQL_TCP_PROTOCOL=1']
        : env,
      HostConfig: {
        AutoRemove: true,
        PortBindings: {
          [databaseType === 'sqlserver' ? '1433/tcp' : '3306/tcp']: [
            { HostPort: databaseType === 'sqlserver' ? '1433' : '3306' }
          ]
        },
        ...(databaseType === 'sqlserver' && {
          Memory: 2147483648 // 2GB内存限制
        })
      }
    });

    // 对于 SQL Server，安装 mssql-tools
    if (databaseType === 'sqlserver') {
      await container.start();

      // 安装依赖工具链
      await new Promise((resolve, reject) => {
        const exec = container.exec({
          Cmd: [
            '/bin/bash',
            '-c',
            'echo "YourStrong!Passw0rd" | su -c "apt-get update && apt-get install -y curl && curl https://packages.microsoft.com/keys/microsoft.asc | apt-key add - && curl https://packages.microsoft.com/config/ubuntu/20.04/prod.list > /etc/apt/sources.list.d/mssql-release.list"'
          ],
          AttachStdout: true,
          AttachStderr: true
        }, (err, exec) => {
          if (err) return reject(err);
          exec.start({}, (err, stream) => {
            if (err) return reject(err);
            stream?.on('end', () => resolve(null));
            stream?.resume();
          });
        });
      });

      // 安装核心组件
      await new Promise((resolve, reject) => {
        const exec = container.exec({
          Cmd: [
            '/bin/bash',
            '-c',
            'ACCEPT_EULA=Y apt-get install -y msodbcsql17 mssql-tools && echo "export PATH=$PATH:/opt/mssql-tools/bin" >> ~/.bashrc && source ~/.bashrc'
          ],
          AttachStdout: true,
          AttachStderr: true
        }, (err, exec) => {
          if (err) return reject(err);
          exec.start({}, (err, stream) => {
            if (err) return reject(err);
            stream?.on('end', () => resolve(null));
            stream?.resume();
          });
        });
      });

      await new Promise(resolve => setTimeout(resolve, 5000)); // 新增等待时间
    } else {
      await container.start();
    }

    // 改进的健康检查逻辑
    let isReady = false;
    const startTime = Date.now();
    while (Date.now() - startTime < CONTAINER_TIMEOUT * 1000) {
      try {
        const healthCheckCmd = databaseType === 'sqlserver'
          ? [
              '/opt/mssql-tools18/bin/sqlcmd',
              '-S', 'localhost',
              '-U', 'SA',
              '-P', process.env.SQL_SERVER_SA_PASSWORD,
              '-C',  // 添加信任证书参数
              '-Q', 'SELECT 1'
            ]
          : [
              'sh',
              '-c',
              `mysql -h 127.0.0.1 -u root -p${process.env.MYSQL_ROOT_PASSWORD} -e "SELECT 1" 2>&1 | grep -v "Using a password"`
            ];

        const exec = await container.exec({
          Cmd: healthCheckCmd,
          AttachStdout: true,
          AttachStderr: true
        });

        // 验证健康检查结果
        const output = await new Promise<string>((resolve, reject) => {
          exec.start({}, (err, stream) => {
            if (err) return reject(err);
            let data = '';
            stream?.on('data', (chunk: Buffer) => data += chunk.toString());
            stream?.on('end', () => resolve(data));
            stream?.resume();
          });
        });

        if (output.includes("ERROR")) throw new Error(output);
        isReady = true;
        break;
      } catch {
        await new Promise(resolve => setTimeout(resolve, 2000));
      }
    }

    if (!isReady) throw new Error(`Database not ready within ${CONTAINER_TIMEOUT} seconds`);

    // 执行SQL命令
    const exec = await container.exec({
      Cmd: [
        databaseType === 'sqlserver' ? '/bin/bash' : '/bin/sh',
        '-c',
        command
      ],
      AttachStdout: true,
      AttachStderr: true
    });

    // 捕获执行结果
    return await new Promise((resolve, reject) => {
      let output = '';
      exec.start({}, (err, stream) => {
        if (err) return reject(err);

        stream?.on('data', (chunk: Buffer) => {
          output += chunk.toString();
        });

        stream?.on('end', () => {
          if (databaseType === 'mysql') {
            output = output.replace(/Warning: Using a password on the command line interface can be insecure.\n/g, '');
          }
          resolve(output);
        });

        stream?.resume();
      });
    });
  } catch (error) {
    // 异常清理
    if (container) {
      await container.stop().catch(() => {});
      await container.remove().catch(() => {});
    }
    throw typeof error === 'string'
        ? new Error(error)
        : error instanceof Error
            ? error
            : new Error('Unknown error');
  }
}
